from ydnatl.tags.tag_factory import simple_tag_class

H1 = simple_tag_class("h1")
H2 = simple_tag_class("h2")
H3 = simple_tag_class("h3")
H4 = simple_tag_class("h4")
H5 = simple_tag_class("h5")
H6 = simple_tag_class("h6")
Paragraph = simple_tag_class("p")
Blockquote = simple_tag_class("blockquote")
Pre = simple_tag_class("pre")
Quote = simple_tag_class("q")
Cite = simple_tag_class("cite")
Em = simple_tag_class("em")
Italic = simple_tag_class("i")
Span = simple_tag_class("span")
Strong = simple_tag_class("strong")
Bold = simple_tag_class("b")
Abbr = simple_tag_class("abbr")
Link = simple_tag_class("a")
Small = simple_tag_class("small")
Superscript = simple_tag_class("sup")
Subscript = simple_tag_class("sub")
Time = simple_tag_class("time")
Code = simple_tag_class("code")
Del = simple_tag_class("del")
Ins = simple_tag_class("ins")
Strikethrough = simple_tag_class("s")
Underline = simple_tag_class("u")
Kbd = simple_tag_class("kbd")
Samp = simple_tag_class("samp")
Var = simple_tag_class("var")
Mark = simple_tag_class("mark")
Dfn = simple_tag_class("dfn")
Br = simple_tag_class("br", self_closing=True)
Wbr = simple_tag_class("wbr", self_closing=True)
